import unittest
import pathlib
from helpers import *

import grole

class TestGrole(unittest.TestCase):

    def setUp(self):
        self.app = grole.Grole()

    def test_hello(self):
        @self.app.route('/')
        def hello(env, req):
            return 'Hello, World!'

        rd = FakeReader(data=b'GET / HTTP/1.1\r\n\r\n')
        wr = FakeWriter()
        a_wait(self.app._handle(rd, wr))
        data = wr.data.split(b'\r\n\r\n')[1]
        self.assertEqual(b'Hello, World!', data)

    def test_async(self):
        @self.app.route('/')
        async def hello_async(env, req):
            return 'Hello, World!'

        rd = FakeReader(data=b'GET / HTTP/1.1\r\n\r\n')
        wr = FakeWriter()
        a_wait(self.app._handle(rd, wr))
        data = wr.data.split(b'\r\n\r\n')[1]
        self.assertEqual(b'Hello, World!', data)

    def test_error(self):
        @self.app.route('/')
        def error(env, req):
            a = []
            return a[1]

        rd = FakeReader(data=b'GET / HTTP/1.1\r\n\r\n')
        wr = FakeWriter()
        a_wait(self.app._handle(rd, wr))
        data = wr.data.split(b'\r\n')[0]
        self.assertEqual(b'HTTP/1.1 500 Internal Server Error', data)

    def test_close_big_error(self):
        @self.app.route('/')
        def error(env, req):
            a = []
            return a[1]

        rd = FakeReader(data=b'GET / HTTP/1.1\r\n\r\n')
        wr = ErrorWriter()
        a_wait(self.app._handle(rd, wr))
        self.assertTrue(wr.closed)


    def test_404(self):
        rd = FakeReader(data=b'GET / HTTP/1.1\r\n\r\n')
        wr = FakeWriter()

        a_wait(self.app._handle(rd, wr))
        data = wr.data.split(b'\r\n')[0]
        self.assertEqual(b'HTTP/1.1 404 Not Found', data)

class TestStatic(unittest.TestCase):

    def setUp(self):
        self.app = grole.Grole()

    def test_file(self):
        testfolder = str(pathlib.Path(__file__).parents[0])
        grole.serve_static(self.app, '', testfolder, index=False)
        rd = FakeReader(data=b'GET /test.dat HTTP/1.1\r\n\r\n')
        wr = FakeWriter()
        a_wait(self.app._handle(rd, wr))
        data = wr.data.split(b'\r\n\r\n')[1]
        self.assertEqual(b'4\r\nfoo\n\r\n0', data)

    def test_index(self):
        testfolder = str(pathlib.Path(__file__).parents[0])
        grole.serve_static(self.app, '', testfolder, index=True)
        rd = FakeReader(data=b'GET / HTTP/1.1\r\n\r\n')
        wr = FakeWriter()
        a_wait(self.app._handle(rd, wr))
        data = wr.data.split(b'\r\n')[0]
        self.assertEqual(b'HTTP/1.1 200 OK', data)

    def test_index2(self):
        testfolder = str(pathlib.Path(__file__).parents[1])
        grole.serve_static(self.app, '', testfolder, index=True)
        rd = FakeReader(data=b'GET /test HTTP/1.1\r\n\r\n')
        wr = FakeWriter()
        a_wait(self.app._handle(rd, wr))
        data = wr.data.split(b'\r\n')[0]
        self.assertEqual(b'HTTP/1.1 200 OK', data)

    def test_notfound(self):
        testfolder = str(pathlib.Path(__file__).parents[0])
        grole.serve_static(self.app, '', testfolder, index=False)
        rd = FakeReader(data=b'GET /foo HTTP/1.1\r\n\r\n')
        wr = FakeWriter()
        a_wait(self.app._handle(rd, wr))
        data = wr.data.split(b'\r\n')[0]
        self.assertEqual(b'HTTP/1.1 404 Not Found', data)

class TestDoc(unittest.TestCase):

    def setUp(self):
        self.app = grole.Grole()

    def test_doc(self):
        @self.app.route('/hello')
        def hello(env, req):
            return 'Hello, World!'

        grole.serve_doc(self.app, '/')
        rd = FakeReader(data=b'GET / HTTP/1.1\r\n\r\n')
        wr = FakeWriter()
        a_wait(self.app._handle(rd, wr))
        data = wr.data.split(b'\r\n')[0]
        self.assertEqual(b'HTTP/1.1 200 OK', data)

if __name__ == '__main__':
    unittest.main()



import asyncio
import io

a_wait = asyncio.new_event_loop().run_until_complete

class FakeReader():
    def __init__(self, data=b''):
        self.io = io.BytesIO(data)
        self.len = len(data)

    async def readline(self):
        return self.io.readline()

    async def readexactly(self, n):
        data = self.io.read(n)
        if len(data) != n:
            raise asyncio.IncompleteReadError(data, n)
        return data

    def at_eof(self):
        return self.io.tell() == self.len

class FakeWriter():
    def __init__(self):
        self.data = b''

    async def drain(self):
        return

    def write(self, data):
        self.data += data

    def get_extra_info(self, arg):
        return 'fake'

class ErrorWriter():
    def __init__(self):
        self.closed = False

    async def drain(self):
        return

    def write(self, data):
        raise Exception('Broken')

    def get_extra_info(self, arg):
        return 'fake'

    def close(self):
        self.closed = True



import unittest
import asyncio
import io
import multiprocessing
import urllib.request
import time

import grole

def simple_server():
    app = grole.Grole()

    @app.route('/')
    def hello(env, req):
        return 'Hello, World!'

    app.run(host='127.0.0.1')

class TestServe(unittest.TestCase):

    def test_simple(self):
        p = multiprocessing.Process(target=simple_server)
        p.start()
        time.sleep(0.1)
        with urllib.request.urlopen('http://127.0.0.1:1234') as response:
               html = response.read()
               self.assertEqual(html, b'Hello, World!')
        p.terminate()

    def test_fileserver(self):
        p = multiprocessing.Process(target=grole.main, args=[['-a', '127.0.0.1']])
        p.start()
        time.sleep(0.1)
        with urllib.request.urlopen('http://127.0.0.1:1234/test/test.dat') as response:
               html = response.read()
               self.assertEqual(html, b'foo\n')
        p.terminate()

    def test_https(self):
        p = multiprocessing.Process(target=simple_server)
        p.start()
        time.sleep(0.1)
        self.assertRaises(urllib.error.URLError)
        p.terminate()





#!/usr/bin/env python3
"""
Grole is a python (3.5+) nano web framework based on asyncio. It's goals are to be simple, embedable (single file and standard library only) and easy to use.
"""
import asyncio
import socket
import json
import re
import urllib
import traceback
import inspect
import io
import mimetypes
import pathlib
import html
import sys
import argparse
import logging
from collections import defaultdict

__author__ = 'witchard'
__version__ = '0.2.1'

class Request:
    """
    Represents a single HTTP request

    The following members are populated with the request details:

      * method:   The request method
      * location: The request location as it is sent
      * path:     The unescaped path part of the location
      * query:    The query string part of the location (if present)
      * version:  The request version, e.g. HTTP/1.1
      * headers:  Dictionary of headers from the request
      * data:     Raw data from the request body
      * match:    The re.MatchObject from the successful path matching 
    """

    async def _read(self, reader):
        """
        Parses HTTP request into member variables
        """
        start_line = await self._readline(reader)
        self.method, self.location, self.version = start_line.decode().split()
        path_query = self.location.split('?', 1)
        self.path = urllib.parse.unquote(path_query[0])
        self.query = {}
        if len(path_query) > 1:
            for q in path_query[1].split('&'):
                try:
                    k, v = q.split('=', 1)
                    self.query[k] = v
                except ValueError:
                    self.query[q] = None
        self.headers = {}
        while True:
            header_raw = await self._readline(reader)
            if header_raw.strip() == b'':
                break
            header = header_raw.decode().split(':', 1)
            self.headers[header[0]] = header[1].strip()

        # TODO implement chunked handling
        self.data = b''
        await self._buffer_body(reader)

    async def _readline(self, reader):
        """
        Readline helper
        """
        ret = await reader.readline()
        if len(ret) == 0 and reader.at_eof():
            raise EOFError()
        return ret

    async def _buffer_body(self, reader):
        """
        Buffers the body of the request
        """
        remaining = int(self.headers.get('Content-Length', 0))
        if remaining > 0:
            try:
                self.data = await reader.readexactly(remaining)
            except asyncio.IncompleteReadError:
                raise EOFError()

    def body(self):
        """
        Decodes body as string
        """
        return self.data.decode()

    def json(self):
        """
        Decodes json object from the body
        """
        return json.loads(self.body())

class ResponseBody:
    """
    Response body from a byte string

    """
    def __init__(self, data=b'', content_type='text/plain'):
        """
        Initialise object, data is the data to send
    
        Parameters:

            * data: Byte data to send
            * content_type: Value of Content-Type header, default text/plain
        """
        self._headers = {'Content-Length': len(data),
                         'Content-Type': content_type}
        self._data = data

    def _set_headers(self, headers):
        """
        Merge internal headers into the passed in dictionary
        """
        headers.update(self._headers)

    async def _write(self, writer):
        """
        Write out the data
        """
        writer.write(self._data)
        await writer.drain()

class ResponseString(ResponseBody):
    """
    Response body from a string
    """
    def __init__(self, data='', content_type='text/plain'):
        """
        Initialise object, data is the data to send
    
        Parameters:

            * data: String data to send
            * content_type: Value of Content-Type header, default text/plain
        """
        super().__init__(data.encode(), content_type)

class ResponseJSON(ResponseString):
    """
    Response body encoded in json
    """
    def __init__(self, data='', content_type='application/json'):
        """
        Initialise object, data is the data to send
    
        Parameters:

            * data: Object to encode as json for sending
            * content_type: Value of Content-Type header, default application/json
        """
        super().__init__(json.dumps(data), content_type)

class ResponseFile(ResponseBody):
    """
    Respond with a file

    Content type is guessed if not provided
    """
    def __init__(self, filename, content_type=None):
        """
        Initialise object, data is the data to send
    
        Parameters:

            * filename: Name of file to read and send
            * content_type: Value of Content-Type header, default is to guess from file extension
        """
        if content_type == None:
            content_type = mimetypes.guess_type(filename)[0]
        self.filename = filename
        self._headers = {'Transfer-Encoding': 'chunked',
                         'Content-Type': content_type}

    async def _write(self, writer):
        f = io.FileIO(self.filename)
        while True:
            data = f.read(io.DEFAULT_BUFFER_SIZE)
            header = format(len(data), 'x') + '\r\n'
            writer.write(header.encode())
            writer.write(data)
            writer.write(b'\r\n')
            await writer.drain()
            if len(data) == 0:
                f.close()
                return # EOF

class Response:
    """
    Represents a single HTTP response
    """
    def __init__(self, data=None, code=200, reason='OK', headers={},
                 version='HTTP/1.1'):
        """
        Create a response

        Parameters:

          * data:     Object to send e.g. ResponseBody / ResponseJSON.
          * code:     The response code, default 200
          * reason:   The response reason, default OK
          * version:  The response version, default HTTP/1.1
          * headers:  Dictionary of response headers, default is a Server header and those from the response body

        Note, data is intelligently converted to an appropriate ResponseXYZ object depending on it's type.
        """
        self.version = version
        self.code = code
        self.reason = reason
        self.data = self._create_body(data)
        self.headers = {'Server': 'grole/' + __version__}
        self.data._set_headers(self.headers) # Update headers from data
        self.headers.update(headers) # Update headers from user

    async def _write(self, writer):
        start_line = '{} {} {}\r\n'.format(self.version, self.code, self.reason)
        headers = ['{}: {}'.format(x[0], x[1]) for x in self.headers.items()] 
        header = start_line + '\r\n'.join(headers) + '\r\n\r\n'
        writer.write(header.encode())
        await writer.drain()
        await self.data._write(writer)

    def _create_body(self, data):
        if isinstance(data, ResponseBody):
            return data
        elif data is None:
            return ResponseBody()
        elif isinstance(data, bytes):
            return ResponseBody(data)
        elif isinstance(data, str):
            return ResponseString(data)
        else:
            return ResponseJSON(data)

def serve_static(app, base_url, base_path, index=False):
    """
    Serve a directory statically

    Parameters:

        * app: Grole application object
        * base_url: Base URL to serve from, e.g. /static
        * base_path: Base path to look for files in
        * index: Provide simple directory indexes if True
    """
    @app.route(base_url + '/(.*)')
    def serve(env, req):
        """
        Static files
        """
        try:
            base = pathlib.Path(base_path).resolve()
            path = (base / req.match.group(1)).resolve()
        except FileNotFoundError:
            return Response(None, 404, 'Not Found')


        # Don't let bad paths through
        if base == path or base in path.parents:
            if path.is_file():
                return ResponseFile(str(path))
            if index and path.is_dir():
                if base == path:
                    ret = ''
                else:
                    ret = '<a href="../">../</a><br/>\r\n'
                for item in path.iterdir():
                    name = item.parts[-1]
                    if item.is_dir():
                        name += '/'
                    ret += '<a href="{}">{}</a><br/>\r\n'.format(urllib.parse.quote(name), html.escape(name))
                ret = ResponseString(ret, 'text/html')
                return ret

        return Response(None, 404, 'Not Found')

def serve_doc(app, url):
    """
    Serve API documentation extracted from request handler docstrings

    Parameters:
        * app: Grole application object
        * url: URL to serve at
    """
    @app.route(url, doc=False)
    def index(env, req):
        ret = ''
        for d in env['doc']:
            ret += 'URL: {url}, supported methods: {methods}{doc}\n'.format(**d)
        return ret

class Grole:
    """
    A Grole Webserver
    """
    def __init__(self, env={}):
        """
        Initialise a server

        env is passed to request handlers to provide shared state.
        Note, env by default contains doc which is populated from
        registered route docstrings.
        """
        self._handlers = defaultdict(list)
        self.env = {'doc': []}
        self.env.update(env)
        self._logger = logging.getLogger('grole')

    def route(self, path_regex, methods=['GET'], doc=True):
        """
        Decorator to register a handler

        Parameters:
            * path_regex: Request path regex to match against for running the handler
            * methods: HTTP methods to use this handler for
            * doc: Add to internal doc structure
        """
        def register_func(func):
            """
            Decorator implementation
            """
            if doc:
                self.env['doc'].append({'url': path_regex, 'methods': ', '.join(methods), 'doc': func.__doc__})
            for method in methods:
                self._handlers[method].append((re.compile(path_regex), func))
            return func # Return the original function
        return register_func # Decorator

    async def _handle(self, reader, writer):
        """
        Handle a single TCP connection

        Parses requests, finds appropriate handlers and returns responses
        """
        peer = writer.get_extra_info('peername')
        self._logger.debug('New connection from {}'.format(peer))
        try:
            # Loop handling requests
            while True:
                # Read the request
                req = Request()
                await req._read(reader)

                # Find and execute handler
                res = None
                for path_regex, handler in self._handlers.get(req.method, []):
                    match = path_regex.fullmatch(req.path)
                    if match:
                        req.match = match
                        try:
                            if inspect.iscoroutinefunction(handler):
                                res = await handler(self.env, req)
                            else:
                                res = handler(self.env, req)
                            if not isinstance(res, Response):
                                res = Response(data=res)
                        except:
                            # Error - log it and return 500
                            self._logger.error(traceback.format_exc())
                            res = Response(code=500, reason='Internal Server Error')
                        break

                # No handler - send 404
                if res == None:
                    res = Response(code=404, reason='Not Found')

                # Respond
                await res._write(writer)
                self._logger.info('{}: {} -> {}'.format(peer, req.path,  res.code))
        except EOFError:
            self._logger.debug('Connection closed from {}'.format(peer))
        except Exception as e:
            self._logger.error('Connection error ({}) from {}'.format(e, peer))
            writer.close()

    def run(self, host='localhost', port=1234):
        """
        Launch the server. Will run forever accepting connections until interrupted.

        Parameters:

            * host: The host to listen on
            * port: The port to listen on
        """
        # Setup loop
        loop = asyncio.get_event_loop()
        coro = asyncio.start_server(self._handle, host, port, loop=loop)
        try:
            server = loop.run_until_complete(coro)
        except Exception as e:
            self._logger.error('Could not launch server: {}'.format(e))
            return

        # Run the server
        self._logger.info('Serving on {}'.format(server.sockets[0].getsockname()))
        try:
            loop.run_forever()
        except KeyboardInterrupt:
            pass

        # Close the server
        server.close()
        loop.run_until_complete(server.wait_closed())
        loop.close()

def parse_args(args=sys.argv[1:]):
    """
    Parse command line arguments for Grole server running as static file server
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('-a', '--address', help='address to listen on, default localhost',
                                default='localhost')
    parser.add_argument('-p', '--port', help='port to listen on, default 1234',
                                default=1234, type=int)
    parser.add_argument('-d', '--directory', help='directory to serve, default .',
                                default='.')
    parser.add_argument('-n', '--noindex', help='do not show directory indexes',
                                default=False, action='store_true')
    loglevel = parser.add_mutually_exclusive_group()
    loglevel.add_argument('-v', '--verbose', help='verbose logging',
                                default=False, action='store_true')
    loglevel.add_argument('-q', '--quiet', help='quiet logging',
                                default=False, action='store_true')
    return parser.parse_args(args)

def main(args=sys.argv[1:]):
    """
    Run Grole static file server
    """
    args = parse_args(args)
    if args.verbose:
        logging.basicConfig(level=logging.DEBUG)
    elif args.quiet:
        logging.basicConfig(level=logging.ERROR)
    else:
        logging.basicConfig(level=logging.INFO)
    app = Grole()
    serve_static(app, '', args.directory, not args.noindex)
    app.run(args.address, args.port)

if __name__ == '__main__':
    main()



import unittest

import grole

class TestArgs(unittest.TestCase):

    def test_defaults(self):
        args = grole.parse_args([])
        self.assertEqual(args.address, 'localhost')
        self.assertEqual(args.port, 1234)
        self.assertEqual(args.directory, '.')
        self.assertEqual(args.noindex, False)
        self.assertEqual(args.verbose, False)
        self.assertEqual(args.quiet, False)

    def test_override(self):
        args = grole.parse_args(['-a', 'foo', '-p', '27', '-d', 'bar', '-n', '-v'])
        self.assertEqual(args.address, 'foo')
        self.assertEqual(args.port, 27)
        self.assertEqual(args.directory, 'bar')
        self.assertEqual(args.noindex, True)
        self.assertEqual(args.verbose, True)
        self.assertEqual(args.quiet, False)

    def test_error(self):
        try:
            grole.parse_args(['-q', '-v'])
        except SystemExit:
            return
        self.fail('Did not error on mutually exclusive args')



import unittest

import grole

class TestMain(unittest.TestCase):

    def test_launch(self):
        # Success is that it doesn't do anything
        grole.main(['-p', '80'])

    def test_launch2(self):
        # Success is that it doesn't do anything
        grole.main(['-p', '80', '-q'])

    def test_launch3(self):
        # Success is that it doesn't do anything
        grole.main(['-p', '80', '-v'])



import unittest
import pathlib
from helpers import FakeWriter, a_wait

import grole

class TestHeader(unittest.TestCase):

    def test_header(self):
        res = grole.Response(None, 123, 'foo', {'foo': 'bar'}, 'bar')
        writer = FakeWriter()
        a_wait(res._write(writer))
        for line in writer.data.split(b'\r\n'):
            if line.startswith(b'bar'):
                self.assertEqual(line, b'bar 123 foo')
            elif line.startswith(b'Content-Type'):
                self.assertEqual(line, b'Content-Type: text/plain')
            elif line.startswith(b'Content-Length'):
                self.assertEqual(line, b'Content-Length: 0')
            elif line.startswith(b'foo'):
                self.assertEqual(line, b'foo: bar')
            elif line.startswith(b'Server'):
                self.assertEqual(line, b'Server: grole/' + grole.__version__.encode())
            else:
                if line != b'':
                    self.fail('Extra data: ' + line.decode())

class TestBody(unittest.TestCase):

    def test_headers(self):
        res = grole.ResponseBody(b'foo', content_type='bar')
        hdr = {}
        res._set_headers(hdr)
        self.assertDictEqual(hdr, {'Content-Length': 3,
                                   'Content-Type': 'bar'})

    def test_data(self):
        res = grole.ResponseBody(b'foo', content_type='bar')
        writer = FakeWriter()
        a_wait(res._write(writer))
        self.assertEqual(writer.data, b'foo')

    def test_bytes(self):
        res = grole.Response(b'foo')
        self.assertIsInstance(res.data, grole.ResponseBody)

    def test_string(self):
        res = grole.Response('foo')
        self.assertIsInstance(res.data, grole.ResponseString)

    def test_json(self):
        res = grole.Response(['foo'])
        self.assertIsInstance(res.data, grole.ResponseJSON)

    def test_file(self):
        res = grole.Response(grole.ResponseFile('foo'))
        self.assertIsInstance(res.data, grole.ResponseFile)

class TestString(unittest.TestCase):

    def setUp(self):
        self.res = grole.ResponseString('foo', content_type='bar')

    def test_headers(self):
        hdr = {}
        self.res._set_headers(hdr)
        self.assertDictEqual(hdr, {'Content-Length': 3,
                                   'Content-Type': 'bar'})

    def test_data(self):
        writer = FakeWriter()
        a_wait(self.res._write(writer))
        self.assertEqual(writer.data, b'foo')

class TestJSON(unittest.TestCase):

    def setUp(self):
        self.res = grole.ResponseJSON({'foo': 'bar'}, content_type='baz')

    def test_headers(self):
        hdr = {}
        self.res._set_headers(hdr)
        self.assertDictEqual(hdr, {'Content-Length': 14,
                                   'Content-Type': 'baz'})

    def test_data(self):
        writer = FakeWriter()
        a_wait(self.res._write(writer))
        self.assertEqual(writer.data, b'{"foo": "bar"}')

class TestFile(unittest.TestCase):

    def setUp(self):
        testfile = pathlib.Path(__file__).parents[0] / 'test.dat'
        self.res = grole.ResponseFile(str(testfile), content_type='baz')

    def test_headers(self):
        hdr = {}
        self.res._set_headers(hdr)
        self.assertDictEqual(hdr, {'Transfer-Encoding': 'chunked',
                                   'Content-Type': 'baz'})

    def test_data(self):
        writer = FakeWriter()
        a_wait(self.res._write(writer))
        self.assertEqual(writer.data, b'4\r\nfoo\n\r\n0\r\n\r\n')


if __name__ == '__main__':
    unittest.main()



import unittest
from helpers import FakeReader, a_wait

import grole

class TestEncoding(unittest.TestCase):

    def setUp(self):
        self.req = grole.Request()
        self.req.data = b'{"foo": "bar"}'

    def test_body(self):
        self.assertEqual(self.req.body(), '{"foo": "bar"}')

    def test_json(self):
        self.assertEqual(self.req.json(), {'foo': 'bar'})

class TestReading(unittest.TestCase):
    
    def setUp(self):
        self.req = grole.Request()

    def test_readline_returns_data(self):
        reader = FakeReader(b'foo\r\nnope')
        line = a_wait(self.req._readline(reader))
        self.assertEqual(line, b'foo\r\n')

    def test_readline_raises_eof(self):
        reader = FakeReader(b'')
        with self.assertRaises(EOFError):
            line = a_wait(self.req._readline(reader))

    def test_buffer_body_content_len_0(self):
        reader = FakeReader(b'foo')
        self.req.headers = {'Content-Length': 0 }
        self.req.data = b''
        a_wait(self.req._buffer_body(reader))
        self.assertEqual(b'', self.req.data)

    def test_buffer_body_content(self):
        reader = FakeReader(b'foobar')
        self.req.headers = {'Content-Length': 3 }
        self.req.data = b''
        a_wait(self.req._buffer_body(reader))
        self.assertEqual(b'foo', self.req.data)

    def test_buffer_body_not_enough_data(self):
        reader = FakeReader(b'foo')
        self.req.headers = {'Content-Length': 4 }
        self.req.data = b''
        with self.assertRaises(EOFError):
            a_wait(self.req._buffer_body(reader))

    def test_header(self):
        header = b'\r\n'.join([b'GET /foo?bar=baz&spam=eggs&chips HTTP/1.1',
                               b'foo: bar',
                               b'']) + b'\r\n'
        a_wait(self.req._read(FakeReader(header)))
        self.assertEqual(self.req.method, 'GET')
        self.assertEqual(self.req.version, 'HTTP/1.1')
        self.assertEqual(self.req.path, '/foo')
        self.assertEqual(self.req.query, {'bar': 'baz', 'spam': 'eggs', 'chips': None})
        self.assertEqual(self.req.headers, {'foo': 'bar'})
        self.assertEqual(self.req.data, b'')

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

import sys
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.version_info < (3, 5):
    raise NotImplementedError("Sorry, you need at least Python Python 3.5+ to use grole.")

import grole

setup(name='grole',
      version=grole.__version__,
      description='A simple asyncio based web framework',
      long_description=grole.__doc__,
      author=grole.__author__,
      author_email='witchard@hotmail.co.uk',
      url='https://github.com/witchard/grole',
      download_url='https://github.com/witchard/grole/tarball/' + grole.__version__,
      py_modules=['grole'],
      scripts=['grole.py'],
      license='MIT',
      platforms='any',
      keywords=['web', 'framework', 'asyncio', 'grole'],
      classifiers=['Development Status :: 4 - Beta',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: MIT License',
                   'Topic :: Internet :: WWW/HTTP :: HTTP Servers',
                   'Topic :: Software Development :: Libraries :: Application Frameworks',
                   'Programming Language :: Python :: 3.5',
                   'Programming Language :: Python :: 3.6',
                  ],
     )
